@@ -1,8 +1,11 @@
+// Package api provides functionalities for interacting with the API.
 package api
 
 import (
 	"bytes"
+	"context"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"io"
 	"net/http"
@@ -13,17 +16,17 @@ import (
 
 	"github.com/prometheus/client_golang/prometheus"
 	"github.com/prometheus/client_golang/prometheus/promauto"
-
-	"gitlab.com/gitlab-org/gitaly/v15/proto/go/gitalypb"
+	"gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypb"
 
 	"gitlab.com/gitlab-org/gitlab/workhorse/internal/config"
 	"gitlab.com/gitlab-org/gitlab/workhorse/internal/helper"
+	"gitlab.com/gitlab-org/gitlab/workhorse/internal/helper/fail"
 	"gitlab.com/gitlab-org/gitlab/workhorse/internal/log"
 	"gitlab.com/gitlab-org/gitlab/workhorse/internal/secret"
 )
 
 const (
-	// Custom content type for API responses, to catch routing / programming mistakes
+	// ResponseContentType for API responses, to catch routing / programming mistakes
 	ResponseContentType = "application/vnd.gitlab-workhorse+json"
 
 	failureResponseLimit = 32768
@@ -31,12 +34,25 @@ const (
 	geoProxyEndpointPath = "/api/v4/geo/proxy"
 )
 
+var errResponseLimit = fmt.Errorf("response body exceeded maximum buffer size (%d bytes)", failureResponseLimit)
+
+// API represents a client for interacting with an external API.
 type API struct {
 	Client  *http.Client
 	URL     *url.URL
 	Version string
 }
 
+// PreAuthorizeFixedPathError represents an error returned when authorization fails due to fixed path.
+type PreAuthorizeFixedPathError struct {
+	StatusCode int
+	Status     string
+}
+
+func (p *PreAuthorizeFixedPathError) Error() string {
+	return fmt.Sprintf("no api response: status %d", p.StatusCode)
+}
+
 var (
 	requestsCounter = promauto.NewCounterVec(
 		prometheus.CounterOpts{
@@ -53,6 +69,7 @@ var (
 	)
 )
 
+// NewAPI creates a new API client with the given URL, version, and round tripper.
 func NewAPI(myURL *url.URL, version string, roundTripper http.RoundTripper) *API {
 	return &API{
 		Client:  &http.Client{Transport: roundTripper},
@@ -61,20 +78,24 @@ func NewAPI(myURL *url.URL, version string, roundTripper http.RoundTripper) *API
 	}
 }
 
+// GeoProxyEndpointResponse represents the response structure for geo-proxy endpoint data.
 type GeoProxyEndpointResponse struct {
 	GeoProxyURL       string `json:"geo_proxy_url"`
 	GeoProxyExtraData string `json:"geo_proxy_extra_data"`
 	GeoEnabled        bool   `json:"geo_enabled"`
 }
 
+// GeoProxyData represents data url, extra data and enabled or not.
 type GeoProxyData struct {
 	GeoProxyURL       *url.URL
 	GeoProxyExtraData string
 	GeoEnabled        bool
 }
 
+// HandleFunc defines the signature of functions used to handle HTTP requests.
 type HandleFunc func(http.ResponseWriter, *http.Request, *Response)
 
+// MultipartUploadParams represents parameters for a multipart upload operation.
 type MultipartUploadParams struct {
 	// PartSize is the exact size of each uploaded part. Only the last one can be smaller
 	PartSize int64
@@ -86,17 +107,21 @@ type MultipartUploadParams struct {
 	AbortURL string
 }
 
+// ObjectStorageParams represents parameters for configuring object storage.
 type ObjectStorageParams struct {
 	Provider      string
 	S3Config      config.S3Config
 	GoCloudConfig config.GoCloudConfig
 }
 
+// RemoteObject represents URLs for getting, deleting and storing objects in an object storage service.
 type RemoteObject struct {
 	// GetURL is an S3 GetObject URL
 	GetURL string
 	// DeleteURL is a presigned S3 RemoveObject URL
 	DeleteURL string
+	// Whether Workhorse needs to delete the temporary object or not.
+	SkipDelete bool
 	// StoreURL is the temporary presigned S3 PutObject URL to which upload the first found file
 	StoreURL string
 	// Boolean to indicate whether to use headers included in PutHeaders
@@ -117,17 +142,35 @@ type RemoteObject struct {
 	ObjectStorage *ObjectStorageParams
 }
 
+// DuoWorkflow holds configuration for the Duo Workflow service.
+type DuoWorkflow struct {
+	Headers    map[string]string
+	ServiceURI string
+	Secure     bool
+}
+
+// Response represents a structure containing various GitLab-related environment variables.
 type Response struct {
 	// GL_ID is an environment variable used by gitlab-shell hooks during 'git
 	// push' and 'git pull'
-	GL_ID string
+	GL_ID string // nolint:revive // used as env variable
 
 	// GL_USERNAME holds gitlab username of the user who is taking the action causing hooks to be invoked
-	GL_USERNAME string
+	GL_USERNAME string // nolint:revive // used as env variable
 
 	// GL_REPOSITORY is an environment variable used by gitlab-shell hooks during
 	// 'git push' and 'git pull'
-	GL_REPOSITORY string
+	GL_REPOSITORY string // nolint:revive // used as env variable
+
+	// Id of the pipeline build
+	GLBuildID string
+
+	// Id of the requested project
+	ProjectID int
+
+	// Id of the requested project's root namespace
+	RootNamespaceID int
+
 	// GitConfigOptions holds the custom options that we want to pass to the git command
 	GitConfigOptions []string
 	// StoreLFSPath is provided by the GitLab Rails application to mark where the tmp file should be placed.
@@ -159,12 +202,22 @@ type Response struct {
 	ProcessLsif bool
 	// The maximum accepted size in bytes of the upload
 	MaximumSize int64
+	// A list of permitted hash functions. If empty, then all available are permitted.
+	UploadHashFunctions []string
+	// NeedAudit indicates whether git events should be audited to rails.
+	NeedAudit bool `json:"NeedAudit"`
+	// Gob contains settings for the GitLab Observability Backend (GOB).
+	Gob GOBSettings `json:"gob"`
+
+	DuoWorkflow *DuoWorkflow
 }
 
+// GitalyServer represents configuration parameters for a Gitaly server,
+// including its address, access token, and additional call metadata.
 type GitalyServer struct {
-	Address  string            `json:"address"`
-	Token    string            `json:"token"`
-	Features map[string]string `json:"features"`
+	Address      string            `json:"address"`
+	Token        string            `json:"token"`
+	CallMetadata map[string]string `json:"call_metadata"`
 }
 
 // singleJoiningSlash is taken from reverseproxy.go:singleJoiningSlash
@@ -207,25 +260,25 @@ func joinURLPath(a *url.URL, b string) (path string, rawpath string) {
 }
 
 // rebaseUrl is taken from reverseproxy.go:NewSingleHostReverseProxy
-func rebaseUrl(url *url.URL, onto *url.URL, suffix string) *url.URL {
-	newUrl := *url
-	newUrl.Scheme = onto.Scheme
-	newUrl.Host = onto.Host
-	newUrl.Path, newUrl.RawPath = joinURLPath(url, suffix)
+func rebaseURL(url *url.URL, onto *url.URL, suffix string) *url.URL {
+	newURL := *url
+	newURL.Scheme = onto.Scheme
+	newURL.Host = onto.Host
+	newURL.Path, newURL.RawPath = joinURLPath(url, suffix)
 
-	if onto.RawQuery == "" || newUrl.RawQuery == "" {
-		newUrl.RawQuery = onto.RawQuery + newUrl.RawQuery
+	if onto.RawQuery == "" || newURL.RawQuery == "" {
+		newURL.RawQuery = onto.RawQuery + newURL.RawQuery
 	} else {
-		newUrl.RawQuery = onto.RawQuery + "&" + newUrl.RawQuery
+		newURL.RawQuery = onto.RawQuery + "&" + newURL.RawQuery
 	}
-	return &newUrl
+	return &newURL
 }
 
-func (api *API) newRequest(r *http.Request, suffix string) (*http.Request, error) {
+func (api *API) newRequest(r *http.Request, suffix string) *http.Request {
 	authReq := &http.Request{
 		Method: r.Method,
-		URL:    rebaseUrl(r.URL, api.URL, suffix),
-		Header: helper.HeaderClone(r.Header),
+		URL:    rebaseURL(r.URL, api.URL, suffix),
+		Header: r.Header.Clone(),
 	}
 
 	authReq = authReq.WithContext(r.Context())
@@ -257,7 +310,46 @@ func (api *API) newRequest(r *http.Request, suffix string) (*http.Request, error
 	// requests not going through gitlab-workhorse.
 	authReq.Host = r.Host
 
-	return authReq, nil
+	return authReq
+}
+
+// GitAuditEventRequest represents a request for auditing a Git events.
+type GitAuditEventRequest struct {
+	Action        string                                  `json:"action"`
+	Protocol      string                                  `json:"protocol"`
+	Repo          string                                  `json:"gl_repository"`
+	Username      string                                  `json:"username"`
+	PackfileStats *gitalypb.PackfileNegotiationStatistics `json:"packfile_stats,omitempty"`
+	Changes       string                                  `json:"changes"`
+}
+
+// SendGitAuditEvent sends a Git audit event using the API client.
+func (api *API) SendGitAuditEvent(ctx context.Context, body GitAuditEventRequest) error {
+	b, err := json.Marshal(body)
+	if err != nil {
+		return fmt.Errorf("failed to marshal GitAuditEventRequest: %w", err)
+	}
+
+	auditURL := *api.URL
+	auditURL.Path = "/api/v4/internal/shellhorse/git_audit_event"
+	auditReq, err := http.NewRequest(http.MethodPost, auditURL.String(), bytes.NewReader(b))
+	if err != nil {
+		return fmt.Errorf("failed to create request: %w", err)
+	}
+	auditReq.Header.Set("User-Agent", "GitLab-Workhorse")
+	auditReq.Header.Set("Content-Type", "application/json")
+	auditReq = auditReq.WithContext(ctx)
+	httpResponse, err := api.doRequestWithoutRedirects(auditReq)
+	if err != nil {
+		return fmt.Errorf("SendGitAuditEvent: do request: %v", err)
+	}
+	defer func() { _ = httpResponse.Body.Close() }()
+
+	if httpResponse.StatusCode != http.StatusOK {
+		return fmt.Errorf("SendGitAuditEvent: response status: %s", httpResponse.Status)
+	}
+
+	return nil
 }
 
 // PreAuthorize performs a pre-authorization check against the API for the given HTTP request
@@ -266,7 +358,7 @@ func (api *API) newRequest(r *http.Request, suffix string) (*http.Request, error
 //
 // Only upon successful authorization do we return a non-nil *Response
 func (api *API) PreAuthorize(suffix string, r *http.Request) (_ *http.Response, _ *Response, err error) {
-	authReq, err := api.newRequest(r, suffix)
+	authReq := api.newRequest(r, suffix)
 	if err != nil {
 		return nil, nil, fmt.Errorf("preAuthorizeHandler newUpstreamRequest: %v", err)
 	}
@@ -277,7 +369,7 @@ func (api *API) PreAuthorize(suffix string, r *http.Request) (_ *http.Response,
 	}
 	defer func() {
 		if err != nil {
-			httpResponse.Body.Close()
+			_ = httpResponse.Body.Close()
 		}
 	}()
 	requestsCounter.WithLabelValues(strconv.Itoa(httpResponse.StatusCode), authReq.Method).Inc()
@@ -306,7 +398,7 @@ func (api *API) PreAuthorizeFixedPath(r *http.Request, method string, path strin
 	if err != nil {
 		return nil, fmt.Errorf("construct auth request: %w", err)
 	}
-	authReq.Header = helper.HeaderClone(r.Header)
+	authReq.Header = r.Header.Clone()
 	authReq.URL.RawQuery = r.URL.RawQuery
 
 	failureResponse, apiResponse, err := api.PreAuthorize(path, authReq)
@@ -317,24 +409,32 @@ func (api *API) PreAuthorizeFixedPath(r *http.Request, method string, path strin
 	// We don't need the contents of failureResponse but we are responsible
 	// for closing it. Part of the reason PreAuthorizeFixedPath exists is to
 	// hide this awkwardness.
-	failureResponse.Body.Close()
+	if err = failureResponse.Body.Close(); err != nil {
+		fmt.Printf("Error closing failureResponse body: %s", err)
+	}
 
 	if apiResponse == nil {
-		return nil, fmt.Errorf("no api response: status %d", failureResponse.StatusCode)
+		return nil, &PreAuthorizeFixedPathError{StatusCode: failureResponse.StatusCode, Status: failureResponse.Status}
 	}
 
 	return apiResponse, nil
 }
 
+// PreAuthorizer provides methods for pre-authorizing multipart requests.
+type PreAuthorizer interface {
+	PreAuthorizeHandler(next HandleFunc, suffix string) http.Handler
+}
+
+// PreAuthorizeHandler creates an HTTP handler that pre-authorizes requests.
 func (api *API) PreAuthorizeHandler(next HandleFunc, suffix string) http.Handler {
 	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
 		httpResponse, authResponse, err := api.PreAuthorize(suffix, r)
 		if httpResponse != nil {
-			defer httpResponse.Body.Close()
+			defer func() { _ = httpResponse.Body.Close() }()
 		}
 
 		if err != nil {
-			helper.Fail500(w, r, err)
+			fail.Request(w, r, err)
 			return
 		}
 
@@ -345,7 +445,7 @@ func (api *API) PreAuthorizeHandler(next HandleFunc, suffix string) http.Handler
 			return
 		}
 
-		httpResponse.Body.Close() // Free up the Puma thread
+		defer func() { _ = httpResponse.Body.Close() }()
 
 		copyAuthHeader(httpResponse, w)
 
@@ -360,7 +460,7 @@ func (api *API) doRequestWithoutRedirects(authReq *http.Request) (*http.Response
 }
 
 // removeConnectionHeaders removes hop-by-hop headers listed in the "Connection" header of h.
-// See https://tools.ietf.org/html/rfc7230#section-6.1
+// See https://www.rfc-editor.org/rfc/rfc7230#section-6.1
 func removeConnectionHeaders(h http.Header) {
 	for _, f := range h["Connection"] {
 		for _, sf := range strings.Split(f, ",") {
@@ -382,6 +482,13 @@ func copyAuthHeader(httpResponse *http.Response, w http.ResponseWriter) {
 	}
 }
 
+// These are internal headers that may be returned by the rails backend. They should not be passed back to the client.
+var sensitiveHeaders = map[string]bool{
+	"gitlab-workhorse-send-data": true,
+	"gitlab-sv":                  true,
+	"gitlab-lb":                  true,
+}
+
 func passResponseBack(httpResponse *http.Response, w http.ResponseWriter, r *http.Request) {
 	// NGINX response buffering is disabled on this path (with
 	// X-Accel-Buffering: no) but we still want to free up the Puma thread
@@ -389,17 +496,29 @@ func passResponseBack(httpResponse *http.Response, w http.ResponseWriter, r *htt
 	// the entire response body in memory before sending it on.
 	responseBody, err := bufferResponse(httpResponse.Body)
 	if err != nil {
-		helper.Fail500(w, r, err)
+		// A user can issue a git clone command against a URL that doesn't
+		// get handled by the info refs endpoint, resulting in a full-fledged 404
+		// response (i.e. like the one returned in a browser) that's gonna exceed
+		// the response error limit, eventually making Workhorse returning a 500.
+		// Here we intercept such 404s and just return the response code without a body.
+		if errors.Is(err, errResponseLimit) && httpResponse.StatusCode == 404 {
+			fail.Request(w, r, err, fail.WithStatus(httpResponse.StatusCode))
+		} else {
+			fail.Request(w, r, err)
+		}
 		return
 	}
-	httpResponse.Body.Close() // Free up the Puma thread
+
+	if err = httpResponse.Body.Close(); err != nil {
+		fmt.Printf("Error closing response body: %s", err)
+	}
 	bytesTotal.Add(float64(responseBody.Len()))
 
 	for k, v := range httpResponse.Header {
 		// Accommodate broken clients that do case-sensitive header lookup
 		if k == "Www-Authenticate" {
 			w.Header()["WWW-Authenticate"] = v
-		} else {
+		} else if !sensitiveHeaders[k] {
 			w.Header()[k] = v
 		}
 	}
@@ -417,7 +536,7 @@ func bufferResponse(r io.Reader) (*bytes.Buffer, error) {
 	}
 
 	if n == failureResponseLimit {
-		return nil, fmt.Errorf("response body exceeded maximum buffer size (%d bytes)", failureResponseLimit)
+		return nil, errResponseLimit
 	}
 
 	return responseBody, nil
@@ -427,27 +546,28 @@ func validResponseContentType(resp *http.Response) bool {
 	return helper.IsContentType(ResponseContentType, resp.Header.Get("Content-Type"))
 }
 
+// GetGeoProxyData retrieves Geo proxy data from the API client.
 func (api *API) GetGeoProxyData() (*GeoProxyData, error) {
-	geoProxyApiUrl := *api.URL
-	geoProxyApiUrl.Path, geoProxyApiUrl.RawPath = joinURLPath(api.URL, geoProxyEndpointPath)
-	geoProxyApiReq := &http.Request{
+	geoProxyAPIURL := *api.URL
+	geoProxyAPIURL.Path, geoProxyAPIURL.RawPath = joinURLPath(api.URL, geoProxyEndpointPath)
+	geoProxyAPIReq := &http.Request{
 		Method: "GET",
-		URL:    &geoProxyApiUrl,
+		URL:    &geoProxyAPIURL,
 		Header: make(http.Header),
 	}
 
-	httpResponse, err := api.doRequestWithoutRedirects(geoProxyApiReq)
+	httpResponse, err := api.doRequestWithoutRedirects(geoProxyAPIReq)
 	if err != nil {
 		return nil, fmt.Errorf("GetGeoProxyData: do request: %v", err)
 	}
-	defer httpResponse.Body.Close()
+	defer func() { _ = httpResponse.Body.Close() }()
 
 	if httpResponse.StatusCode != http.StatusOK {
 		return nil, fmt.Errorf("GetGeoProxyData: Received HTTP status code: %v", httpResponse.StatusCode)
 	}
 
 	response := &GeoProxyEndpointResponse{}
-	if err := json.NewDecoder(httpResponse.Body).Decode(response); err != nil {
+	if err = json.NewDecoder(httpResponse.Body).Decode(response); err != nil {
 		return nil, fmt.Errorf("GetGeoProxyData: decode response: %v", err)
 	}
 
